/**
 * 
 */


var executionEnv = (function(){
	// summary:
	//		executionEnv provides functions and variables which contain 
	//		information	about the browser runtime
	// returns:
	//		an Object containing execution environment information
	
	// isFile: 
	//		Boolean indicator of file vs http protocol for the browser
	// 
	// pmr: This is the PMR Number that was passed to the URL. If no PMR was passed, a default PMR value is used
	// taxonomy: This is the taxonomy code of the product, for which data collection will be run.
	// symptom: This is the symptom ID for the data collection to be run.
	// collectionName: This is the readable label for the name of the taxonomy Code
	//
	var isFile = location.protocol === "file:";
	
	//Set to true, if we are running within the development Environment.
	//Used to help navigate through 
	var isDevelopmentEnv=checkDevelopmentEnvironment();
	
	console.debug ("Running In Development Environment? "+isDevelopmentEnv);

	//Set defaults
    var pmrNumber= ""; //This is the PMR Number passed as a URl Parameter
    var collectionTaxonomy = ""; //This is the Taxonomy Code, if passed as a URL Parameter
    var scriptID="";     // This is the Script ID. Could be passed, if the user wants to run the Script directly
    var componentID= ""; //This is the Component ID. Optional
    var productID= ""; // This is the product ID . Optional
    var productVersion=""; //This is the version of the product ID . Optional
    var componentRelease=""; //This is the version of the Component. Optional
    var login=""; //This is the login that was passed
    var email=""; //This is the email address that was passed. Optional
    var linkSource=""; //This is the source of the request to run this collector. Products can set this to track marketing information

	
	if(isFile){ //file protocol, assume user should simply double click. All information should be picked up from the file.
		if(isDevelopmentEnv){ //File protocol in the Dev. Environment
			prodInfoFile="services/devEnvProdInfoFile.json"; //In the development Environment, we read this file
		} else {
			prodInfoFile="properties/prodinfo.json";
		}
		
		try {		
			var deferred=dojo.xhrGet({
				url:prodInfoFile,
				handleAs:"json",
				preventCache:true,
				sync:true,
				load: function(response){
					console.debug("The product JSON file loaded successfully. from "+prodInfoFile);
					productJson=response;
				}, 
				error: function(response){
					console.debug(" There was an error loading the product JSON file from location: "+prodInfoFile + ". Error message=" +response);
				}
			});
		}catch(e){
			console.debug("An exception was thrown while reading the prodinfo.json file from location: "+prodInfoFile);		
		}

		  // get the collection archive absolute filename parameter
	     collectionTaxonomy = productJson.taxonomy;
	
	 }else {
	  var port=location.port;
	  console.debug("Port number: "+port);
	  var params = dojo.queryToObject(window.location.search.slice(1));
    
	  //Get the collection archive absolute filename parameter
      collectionTaxonomy = params["taxonomy"];
      scriptID = params["scriptID"];
      pmrNumber = params["pmr"];
      componentID = params["compId"];
      productID = params["prodId"];
      productVersion=params["version"];
      componentRelease=params["release"];
      login=params["login"];
      email=params["email"];
      linkSource=params["linkSource"];
              		
	  if (!collectionTaxonomy) {
		  console.debug("No Collection Taxonomy was passed as query parameter. ");		
		}
		
	  if(!pmrNumber){
		  console.debug("No PMR Number was passed as query parameter. ");	
		  pmrNumber="00000,000,000";
		}

	  if(!scriptID){
		  console.debug("No script ID was passed as query parameter. ");			  
		}
	}
	
	console.debug("The following URL Parameters were passed:");
    console.debug("ScriptID : "+scriptID);
    console.debug("PMR Number: "+pmrNumber);
    console.debug("Taxonomy name: "+collectionTaxonomy);
    console.debug("componentID : "+componentID);
    console.debug("productID: "+productID);
    console.debug("productVersion: "+productVersion);
    console.debug("componentRelease: "+componentRelease);
    console.debug("login: "+login);
    console.debug("email:"+email);
    console.debug("linkSource:"+linkSource);
    
    console.debug("The following values were calculated: ");
    console.debug("Current locale: "+dojo.locale);
    console.debug("isFile protocol?: "+isFile);
    console.debug("developmentEnvironment?: "+isDevelopmentEnv);

	return { // return Object
		isFile: isFile,
		pmrNumber:pmrNumber,		
		taxonomy:collectionTaxonomy,
		scriptID:scriptID,
		isaDevEnv:isDevelopmentEnv,
		componentID:componentID,
		componentRelease:componentRelease,
		productID:productID,
		productVersion:productVersion,
		login:login,
		email:email,
		linkSource:linkSource
	};
})();

	
//This javascript function will be called, when the data collection completes.	
function collectionCompleted(text){
	// summary:
	//    Collection has completed. Publish this event for others (CollectionStatus) to consume.
	//
	console.debug('Collection Completed: '+text);    
	var arrayOfInputs=new Array();
	arrayOfInputs[0]=text;
	
    dojo.publish("CollectionCompleted", arrayOfInputs);	    
}

function uploadComplete(status){
	// summary:
	//    Upload of collection has completed. Publish this event for others (CollectionStatus) to consume.
	//
	console.debug("Running uploadComplete: " + status);
	var statusReturned = dojo.fromJson(status);
	console.debug("some status" + statusReturned);

	
	var statusJSON = {};
	statusJSON.type = "UPLOAD";
	statusJSON.collectionStatus = {};

	
	if (statusReturned.uploadStatus.returnCode=="success") {
		statusJSON.collectionStatus.canceled = false;
		statusJSON.collectionStatus.collectionFailure = false;
		statusJSON.collectionStatus.completionStatus = "NORMAL";
		statusJSON.collectionStatus.message = statusReturned.uploadStatus.message;
		console.debug("Upload successful");
	}
	else if (statusReturned.uploadStatus.returnCode=="canceled") {
		statusJSON.collectionStatus.canceled = true;
		statusJSON.collectionStatus.completionStatus = "CANCELED";
		statusJSON.collectionStatus.message = statusReturned.uploadStatus.message;
		console.debug("Upload cancelled");
	}
	else if (statusReturned.uploadStatus.returnCode=="error") {
		statusJSON.collectionStatus.completionStatus = "ERROR";
		statusJSON.collectionStatus.collectionFailure = true;
		statusJSON.collectionStatus.canceled = false;
		statusJSON.collectionStatus.message = statusReturned.uploadStatus.message;
		console.debug("Upload failed due to error!");
	}
    dojo.publish("UploadCompleted", [dojo.toJson(statusJSON)]);	    
}

function uploadStatusMsg(msg){
	// summary:
	//    Collection has completed. Publish this event for others (CollectionStatus) to consume.
	//
	console.debug("In uploadStatusMsg");

	var msgJSON = {};
	msgJSON.message = msg;

	dojo.publish("UpdateStatusArea", [dojo.toJson(msgJSON)]);
	   
}

function uploadProgress(percent){
	// summary:
	//    Collection has completed. Publish this event for others (CollectionStatus) to consume.
	//
	console.debug("In uploadProgress");
	
	var statusJSON = {};
	statusJSON.progress = percent;

	dojo.publish("UpdateStatusArea", [dojo.toJson(statusJSON)]);
}

function updateProgressBar(percent) {
	
	var statusJSON = {};
	statusJSON.progress = percent;

	dojo.publish("UpdateStatusArea", [dojo.toJson(statusJSON)]);
}

//
function requestInput(text){
	// summary:
	//		Input has been requested from the Back end.
	//
	console.debug("Request Input String: "+text);
	var textJson=dojo.fromJson(text); 
	var arrayOfInputs=new Array();
	arrayOfInputs[0]=textJson;
		
	dojo.publish("InputRequested", arrayOfInputs);		 
}

//This will publish an updateStatusArea event.
function updateStatusArea(text) {
	// summary:
	//		This function updates the status Area, based on information read from the Applet 
	//
	// TODO: Need to write a utility to clean up strings coming from applet
	    console.debug('raw update status message: '+text);
	    //temporarily replace control characters
	    var escapedNL=text.replace("\n", "%n%").replace("\r", "%r%").replace("\t", "%t%");
	    //now escape any remaining backslash characters
	    escapedNL=escapedNL.replace(/\\/gi,'\\\\');
	    //now replace temporary control string with escaped control characters
	    escapedNL=escapedNL.replace("%n%","\\n").replace("%r%","\\r").replace("%t%","\\t");
	    console.debug("Escaped text: "+escapedNL);
	    var removeQuote=escapedNL.replace(/'/gi,' ');	 //some string might contain quotes
	    var convertedText="{'message':'"+removeQuote+"','status':'info','action':'none'}"; // must be JSON to be used in dojo.publish()
	    console.debug('converted status area message: '+convertedText);
	    
		var arrayOfInputs=new Array();
		arrayOfInputs[0]=convertedText;
		
	    dojo.publish("UpdateStatusArea", arrayOfInputs);	    
}


function checkDevelopmentEnvironment() {
	// summary:
	//		Checks to see if we are running in a development Environment or in Production 
	//	    To run in a development Environment, one needs to use the following query param: developmentEnvironment=true on the URL
	// returns:
	//		true: if this is the dev. Environment
	//      false: if this is Production
	//
	
	//var devEnv=false;
	var params = dojo.queryToObject(window.location.search.slice(1));
	var devEnv= params["developmentEnvironment"];
	if(devEnv=='true'){
		devEnv=true;
	}		
	return devEnv;
}

function getCBWidthInPixels(items) {
	//go through list of options and "measure" width of longest string in pixels
	maxWidth=150;
	for (var i=0; i < items.length; i++) {
		ruler = document.getElementById("CBRuler");
		if (ruler){
			ruler.innerHTML = items[i].label;
			maxWidth = (maxWidth >= ruler.offsetWidth) ? maxWidth : ruler.offsetWidth;
			console.debug("label: "+items[i].label);
			console.debug("maxWidth: "+maxWidth);
		}
	}
	
	//small buffer for arrow button ~20px
	maxWidth+=20;
	
	return(maxWidth);
}


function getUserAgent() {
	// summary:
	//		Get the string that describes the browser
	// returns:
	//		A string that describes the browser
	//
  return navigator.userAgent;
}

function setAutoPDMenu(jsonAPD){	
	console.debug("setAutoPDMenu invoked from applet. "+jsonAPD);
    dojo.publish("SetAutoPDMenuJSON", [jsonAPD]);	
}

//Function to update the archive location for after collection
function setArchiveLocation(archiveLoc) {
	console.debug("Setting the archiveLocation from the applet.");
	
	var archiveJSON = {};
	archiveJSON.location = archiveLoc;
	dojo.publish("SetArchiveLocation", [dojo.toJson(archiveJSON)]);
}
